@@ -52,7 +52,7 @@ class Navigator(Checker):
         # TODO NEW VERSION: Opens DM page and creates new DM
         try:
             # LOAD PAGE
-            LOGGER.debug('\n\nLOADING PAGE')
+            LOGGER.debug('LOADING PAGE')
             self.driver.get(ClientUrls.NEW_DM)
             user_div:WebElement = self._find_element(EC.presence_of_element_located((By.XPATH, Paths.USER_DIV)), wait_time=10)
             LOGGER.debug('Page Loaded')
@@ -124,13 +124,27 @@ class Navigator(Checker):
         """
 
         self.driver.get(ClientUrls.SEARCH_TAGS.format(tag))
-        alert: WebElement = self._check_existence(EC.presence_of_element_located((By.XPATH, Paths.PAGE_NOT_FOUND)))
-        if alert:
-            # Tag does not exist
-            raise InvaildTagError(tag=tag)
-        else: 
-            # Operation Successful
+        if self._is_valid_page(ClientUrls.SEARCH_TAGS.format(tag)):
             return True
+        else:
+            raise InvaildTagError(tag)
+
+    
+    def _nav_location(self:'InstaClient', id:str, slug:str):
+        """Navigates to the page of the location specified by
+        the `id` and `slug`.
+
+        Args:
+            id (str): ID of the location to navigate to.
+            slug (str): Slug of the location to navigate to.
+        """
+
+        self.driver.get(ClientUrls.LOCATION_PAGE.format(id, slug))
+        if self._is_valid_page(ClientUrls.LOCATION_PAGE.format(id, slug)):
+            return True
+        else:
+            raise InvaildLocationError(id, slug)
+
 
 
     def _nav_explore(self:'InstaClient'):
