@@ -67,7 +67,7 @@ class Scraper(Component):
         return notifications
 
 
-    @Component._driver_required
+    @Component._login_required
     def get_profile(self:'InstaClient', username:str, context:bool=True) -> Optional['Profile']:
         
         if context and not self.logged_in and None not in (self.username, self.password):
@@ -371,19 +371,33 @@ class Scraper(Component):
  
 
     @Component._login_required
-    def get_followers(self:'InstaClient', user:str, count:int, use_api:bool=True, deep_scrape:Optional[bool]=False, callback_frequency:int=100, callback=None, **callback_args) -> Optional[Union[List[Profile], List[str]]]:
+    def get_followers(self:'InstaClient', user:str, count:int, use_api:bool=True, deep_scrape:Optional[bool]=False, end_cursor:str=None, callback_frequency:int=100, callback=None, **callback_args) -> Optional[Union[List[Profile], List[str]]]:
         """Scrape an instagram user's followers.
 
+        If `use_api` is set to True, you may reach a rate limit after the first 200 requests.
+        Each request can hold up to 50 users, so you will be able to scrape a max of about 10000
+        users, in a matter of 2 minutes. When the rate limit is reached, a cursor will be returned 
+        along with the followers, so you can in a later time resume the scrape, placing that cursor
+        as the value for the `end_cursor` parameter.
+
         Args:
             user (str): User to scrape
-            count (int): Number of followers to scrape
+            count (int): Number of followers to scrape. Insert None
+                to scrape all of the profile's followers.
             use_api (bool): If set to True, the instaclient module will take advantage
                 of instagram graphql requests to scrape followers. Defaults to False.
+            deep_scrape (bool, optional): If set to True, the instaclient module will
+                scrape each profile individually in order to get more information.
+            end_cursor (str, optional): The END CURSOR of the scrape pagination that can be
+                used to resume the scrape from a certain point.
             callback_frequency (int, optional): Number of scraped followers between updates
-            callback (function): Function with no parameters that gets called with the frequency set by ``callback_frequency``. This method must take a ``scraped`` argument.
+            callback (function): Function with no parameters that gets called with the frequency 
+                set by ``callback_frequency``. This method must take a ``scraped`` argument.
 
         Returns:
-            Optional[Union[List[Profile], List[str]]]: List of instagram usernames or of instagram profile objects
+            Union[Optional[Union[List[Profile], List[str]]], Optional[str]]: List of instagram 
+                usernames or of instagram profile objects and the last cursor used for the scrape
+                If the parameter `use_api` is set to True, the second returned value will be None.
 
         Raises:
             NotLoggedInError: Raised if you are not logged into any account
@@ -395,12 +409,13 @@ class Scraper(Component):
         if not profile:
             raise InvalidUserError(user)
 
+        if not count:
+            count = profile.follower_count
+
         followers = list()
         failed = list()
-        last_callback = 0
         finished_warning = False
-
-        start = time.time() # TODO
+        cursor = end_cursor
         
         if not use_api:
             # Nav User Page
@@ -463,20 +478,38 @@ class Scraper(Component):
             except Exception as error:
                 LOGGER.error('ERROR IN SCRAPING FOLLOWERS', exc_info=error)
         else:
-            request = GraphUrls.GRAPH_FIRST_FOLLOWERS.format(QUERY_HASH=QueryHashes.FOLLOWERS_HASH, ID=profile.id)
+            if not cursor:
+                request = GraphUrls.GRAPH_FIRST_FOLLOWERS.format(QUERY_HASH=QueryHashes.FOLLOWERS_HASH, ID=profile.id)
+            else:
+                request = GraphUrls.GRAPH_CURSOR_FOLLOWERS.format(QUERY_HASH=QueryHashes.FOLLOWERS_HASH, ID=profile.id, END_CURSOR=cursor)
+            requests = 1
             looping = True
+            stopping = False
             while looping:
                 result = self._request(request, use_driver=True)
+                requests += 1
 
                 if not result:
                     break
 
                 status = result.get('status')
                 if not status == 'ok':
-                    break
+                    if result.get('message') == 'rate limited':
+                        LOGGER.exception('Rate limit reached. Stopping scrape.')
+                        break
+                    else:
+                        LOGGER.exception(f'The request with cursor {cursor} failed')
+                        break
 
                 data = result['data']['user']['edge_followed_by']
-                # Load users
+
+                # Get Page Info
+                page_info = data['page_info']
+                if not page_info['has_next_page']:
+                    break
+                else:
+                    cursor = page_info['end_cursor'].replace('==', '')
+                    request = GraphUrls.GRAPH_CURSOR_FOLLOWERS.format(QUERY_HASH=QueryHashes.FOLLOWERS_HASH, ID=profile.id, END_CURSOR=cursor)
                 
                 for user_data in data['edges']:
                     user = user_data['node']
@@ -498,27 +531,19 @@ class Scraper(Component):
                         followers.append(follower)
 
                         if len(followers) % callback_frequency == 0:
+                            LOGGER.debug(f'Requests made: {requests}')
                             if callable(callback):
                                 LOGGER.debug('Called Callback')
                                 callback(scraped = followers, **callback_args)
                             else:
                                 LOGGER.info(f'Scraped {len(followers)} followers so far...')
 
-                # Get Page Info
-                page_info = data['page_info']
-                if not page_info['has_next_page']:
-                    break
-                else:
-                    cursor = page_info['end_cursor'].replace('==', '')
-                    request = GraphUrls.GRAPH_CURSOR_FOLLOWERS.format(QUERY_HASH=QueryHashes.FOLLOWERS_HASH, ID=profile.id, END_CURSOR=cursor)
-                    continue
- 
+            LOGGER.debug(f'Requests made: {requests}')
 
-        end = time.time() # TODO
-        LOGGER.info(f'Scraped Followers: Total: {len(followers)}')
+        LOGGER.info(f'Scraped: {len(followers)}')
 
         if not deep_scrape:
-            return followers
+            return followers, cursor
         else:
             LOGGER.info('Deep scraping profiles...')
             # For every shortlink, scrape Post
@@ -533,23 +558,35 @@ class Scraper(Component):
                 except:
                     failed.append(follower)
             LOGGER.warning(f'Failed: {len(failed)}')
-            return profiles
+            return profiles, cursor
 
 
     @Component._login_required
-    def get_following(self:'InstaClient', user:str, count:int, use_api:bool=True, deep_scrape:Optional[bool]=False, callback_frequency:int=100, callback=None, **callback_args) -> Optional[Union[List[Profile], List[str]]]:
+    def get_following(self:'InstaClient', user:str, count:int, use_api:bool=True, deep_scrape:Optional[bool]=False, end_cursor:str=None, callback_frequency:int=100, callback=None, **callback_args) -> Union[Optional[Union[List[Profile], List[str]]], Optional[str]]:
         """Scrape an instagram user's following.
 
+        If `use_api` is set to True, you may reach a rate limit after the first 200 requests.
+        Each request can hold up to 50 users, so you will be able to scrape a max of about 10000
+        users, in a matter of 2 minutes. When the rate limit is reached, a cursor will be returned 
+        along with the followers, so you can in a later time resume the scrape, placing that cursor
+        as the value for the `end_cursor` parameter.
+
         Args:
             user (str): User to scrape
-            count (int): Number of followers to scrape
+            count (int): Number of followers to scrape. Insert
+                None to get all of the profile's following.
             use_api (bool): If set to True, the instaclient module will take advantage
                 of instagram graphql requests to scrape followers. Defaults to False.
+            end_cursor (str, optional): The END CURSOR of the scrape pagination that can be
+                used to resume the scrape from a certain point.
             callback_frequency (int, optional): Number of scraped followers between updates
-            callback (function): Function with no parameters that gets called with the frequency set by ``callback_frequency``. This method must take a ``scraped`` argument.
+            callback (function): Function with no parameters that gets called with the frequency 
+                set by ``callback_frequency``. This method must take a ``scraped`` argument.
 
         Returns:
-            Optional[Union[List[Profile], List[str]]]: List of instagram usernames or of instagram profile objects
+            Union[Optional[Union[List[Profile], List[str]]], Optional[str]]: List of instagram 
+                usernames or of instagram profile objects and the last cursor used for the scrape
+                If the parameter `use_api` is set to True, the second returned value will be None.
 
         Raises:
             NotLoggedInError: Raised if you are not logged into any account
@@ -561,9 +598,12 @@ class Scraper(Component):
         if not profile:
             raise InvalidUserError(user)
 
+        if not count:
+            count = profile.followed_count
+
         following = list()
         failed = list()
-        last_callback = 0
+        cursor = None
         finished_warning = False
 
         start = time.time() # TODO
@@ -629,21 +669,40 @@ class Scraper(Component):
             except Exception as error:
                 LOGGER.error('ERROR IN SCRAPING FOLLOWERS', exc_info=error)
         else:
-            request = GraphUrls.GRAPH_FIRST_FOLLOWING.format(QUERY_HASH=QueryHashes.FOLLOWING_HASH, ID=profile.id)
+            
+            if not end_cursor:
+                request = GraphUrls.GRAPH_FIRST_FOLLOWING.format(QUERY_HASH=QueryHashes.FOLLOWING_HASH, ID=profile.id)
+            else: 
+                request = GraphUrls.GRAPH_CURSOR_FOLLOWING.format(QUERY_HASH=QueryHashes.FOLLOWING_HASH, ID=profile.id, END_CURSOR=end_cursor)
+            requests = 1
             looping = True
             while looping:
                 result = self._request(request, use_driver=True)
+                requests += 1
 
                 if not result:
                     break
 
                 status = result.get('status')
                 if not status == 'ok':
-                    break
-
+                    if result.get('message') == 'rate limited':
+                        LOGGER.exception('Rate Limited. Stopping scrape')
+                        break
+                    else:
+                        LOGGER.exception(f'The request with cursor {cursor} failed')
+                        break
                 data = result['data']['user']['edge_follow']
-                # Load users
                 
+
+                # Get Page Info
+                page_info = data['page_info']
+                if not page_info['has_next_page']:
+                    break
+                else:
+                    cursor = page_info['end_cursor'].replace('==', '')
+                    request = GraphUrls.GRAPH_CURSOR_FOLLOWING.format(QUERY_HASH=QueryHashes.FOLLOWING_HASH, ID=profile.id, END_CURSOR=cursor)
+
+                # Load users
                 for user_data in data['edges']:
                     user = user_data['node']
                     follower = Profile(
@@ -670,21 +729,14 @@ class Scraper(Component):
                             else:
                                 LOGGER.info(f'Scraped {len(following)} following so far...')
 
-                # Get Page Info
-                page_info = data['page_info']
-                if not page_info['has_next_page']:
-                    break
-                else:
-                    cursor = page_info['end_cursor'].replace('==', '')
-                    request = GraphUrls.GRAPH_CURSOR_FOLLOWING.format(QUERY_HASH=QueryHashes.FOLLOWING_HASH, ID=profile.id, END_CURSOR=cursor)
-                    continue
- 
+                
 
-        end = time.time() # TODO
-        LOGGER.info(f'Scraped Followers: Total: {len(following)}')
+            LOGGER.debug(f'Requests made: {requests}')
+ 
+        LOGGER.info(f'Scraped: {len(following)}')
 
         if not deep_scrape:
-            return following
+            return following, cursor
         else:
             LOGGER.info('Deep scraping profiles...')
             # For every shortlink, scrape Post
@@ -699,105 +751,8 @@ class Scraper(Component):
                 except:
                     failed.append(follower)
             LOGGER.warning(f'Failed: {len(failed)}')
-            return profiles
+            return profiles, cursor
 
-        """Scrape an instagram user's following.
-
-        Args:
-            user (str): User to scrape
-            count (int): Number of followers to scrape
-            check_user (bool, optional): If set to True, checks if the `user` is a valid instagram username. Defaults to True.
-            callback_frequency (int, optional): Number of scraped followers between updates
-            callback (function): Function with no parameters that gets called with the frequency set by ``callback_frequency``. This method must take a ``scraped`` argument.
-
-        Returns:
-            Optional[Union[List[Profile], List[str]]]: List of instagram usernames or of instagram profile objects.
-
-        Raises:
-            NotLoggedInError: Raised if you are not logged into any account
-            InvalidUserError: Raised if the user is invalid
-            PrivateAccountError: Raised if the user is a private account
-            NoSuchElementException: Raised if an element is not found when compiling operation.
-        """
-        self._nav_user(user, check_user=check_user)
-        following_btn:WebElement = self._find_element(EC.presence_of_element_located((By.XPATH, Paths.FOLLOWED_BTN)), url=ClientUrls.NAV_USER.format(user))
-        # Click followers btn
-        self._press_button(following_btn)
-        time.sleep(2)
-        LOGGER.debug(f'Got Following page for <{user}>')
-
-        following = list()
-        failed = list()
-        last_callback = 0
-        finished_warning = False
-
-        start = time.time() # TODO
-        
-        try:
-            while len(following) < count:
-                loop = time.time() # TODO
-                LOGGER.debug(f'Starting Scrape Loop. Followers: {len(following)}')
-                
-                scraped_count = len(following)
-                divs = self._find_element(EC.presence_of_all_elements_located((By.XPATH, Paths.FOLLOWER_USER_DIV)), wait_time=2)
-
-                got_elements = time.time() # TODO
-                LOGGER.debug(f'Got Divs in {got_elements - loop}')
-
-                new = 0
-                for div in divs:
-                    try:
-                        username = div.text.split('\n')[0]
-                        if username not in following and username not in('Follow',) and len(following) < count:
-                            following.append(username)
-                            new += 1
-
-                            if (last_callback + new) % callback_frequency == 0:
-                                if callable(callback):
-                                    LOGGER.debug('Called Callback')
-                                    callback(scraped = following, **callback_args)
-
-                    except:
-                        failed.append(div)
-                        pass
-                
-                if len(following) >= count:
-                    break
-
-                if not finished_warning and len(following) == scraped_count:
-                    LOGGER.info('Detected End of Followers Page')
-                    finished_warning = True
-                    time.sleep(3)
-                elif finished_warning:
-                    LOGGER.info('Finished Followers')
-                    break
-                else:
-                    finished_warning = False
-
-                LOGGER.debug('Scroll')
-                self.scroll(mode=self.END_PAGE_SCROLL, times=2, interval=1)
-        except Exception as error:
-            LOGGER.error('ERROR IN SCRAPING FOLLOWERS', exc_info=error)
-                
-
-        end = time.time() # TODO
-        LOGGER.info(f'Scraped Followers: Total: {len(following)}')
-
-        if not deep_scrape:
-            return following
-        else:
-            LOGGER.info('Deep scraping profiles...')
-            # For every shortlink, scrape Post
-            profiles = list()
-            for index, follower in enumerate(following):
-                try:
-                    LOGGER.debug(f'Deep scraped {index} profiles out of {len(following)}')
-                    profiles.append(self.get_profile(follower))
-                except:
-                    failed.append(follower)
-            LOGGER.warning(f'Failed: {len(failed)}')
-            return profiles
-    
     
     # SCRAPE HASHTAG
     @Component._driver_required
